<?php
if (!defined('_PS_VERSION_'))
	exit;

error_reporting(E_ALL);
ini_set('display_errors',1);
class Shipway extends Module{
	
	private $_postErrors = NULL;
	private $_html = '';
	public function __construct(){
		$this->name = 'shipway';
		$this->tab = 'shipping_logistics';
		$this->version = '1.0';
		$this->author = 'Onjection Solutions';
		$this->need_instance = 1;
		$this->ps_versions_compliancy = array('min' => '1.5', 'max' => '1.6');
		$this->bootstrap = true;
		
		parent::__construct();
		
		$this->displayName = $this->l('Shipway');
		$this->description = $this->l('Courier tracking tool.');
		$this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
		
		if (!Configuration::get('SHIPWAY_MODULE_NAME')){
			$this->warning = $this->l('No name provided');			
		}      
	}
	public function install(){
		if (Shop::isFeatureActive()){
			Shop::setContext(Shop::CONTEXT_ALL);
		}
		if(!parent::install() || !Configuration::updateValue('SHIPWAY_MODULE_NAME', 'Shipway Prestashop Module') || !$this->registerHook('actionAdminOrdersTrackingNumberUpdate') ){
			return false;
		}
		
		$db = Db::getInstance();
		
		//creating tables
		$sql = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "sw_couriers` (
				  `carrier_id` int(11) NOT NULL,
				  `name` varchar(255) NOT NULL,
				  status int(2) NOT NULL,
				  PRIMARY KEY (`carrier_id`)
				)";
		$tableCreation = $db->execute($sql);
		
		if(!$tableCreation){
			return false;
		}
		
		
		return true;
	}
	public function uninstall(){
		if(!parent::uninstall() || !Configuration::deleteByName('SHIPWAY_MODULE_NAME') ){
			return false;
		}
		Configuration::deleteByName('SHIPWAY_USERNAME');
		Configuration::deleteByName('SHIPWAY_LINCENCE_KEY');
		Configuration::deleteByName('id_shipway_carrier');
				
		return true;
	}
	public function getContent(){
		
		$this->_postValidation();
		if (!count($this->_postErrors)){
			$this->_postProcess();
		}else{
			foreach ($this->_postErrors as $err){
				$this->_html .= $this->displayError($err);
			}				
		}
			
		
		$this->_html .= $this->renderForm();
		
		if(Configuration::get('SHIPWAY_USERNAME')){
			$this->_html .= $this->renderCarrierForm();
		}
		return $this->_html;
	}
	public function renderForm(){
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Shipway Settings'),
					'icon' => 'icon-cogs'
				),
				'input' => array(
					array(
						'type' => 'text',
						'label' => $this->l('Shipway Username'),
						'name' => 'SHIPWAY_USERNAME',
						'required' => true
					),
					array(
						'type' => 'text',
						'label' => $this->l('Shipway Licence Key'),
						'name' => 'SHIPWAY_LINCENCE_KEY',
						'required' => true
					)
				),
				'submit' => array(
					'title' => $this->l('Save'),
				)
			),
		);
		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table = $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		$this->fields_form = array();
		$helper->id = (int)Tools::getValue('id_carrier');
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'saveShipwaySettings';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'uri' => $this->getPathUri(),
			'fields_value' => $this->getConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);

		return $helper->generateForm(array($fields_form));
	}
	
	public function renderCarrierForm(){
		
		$db = Db::getInstance();
		
		$sql = "SELECT id_carrier,id_reference,`name` FROM `" . _DB_PREFIX_ . "carrier` a WHERE deleted ='0'";
		$storeCarriers = $db->ExecuteS($sql);
		
		$sql = "SELECT `carrier_id`,`name`,`status` FROM " . _DB_PREFIX_ . "sw_couriers WHERE `status` = '1' ";
		$shipwayCarriers = $db->ExecuteS($sql);
		
		if(!empty($storeCarriers) && !empty($shipwayCarriers)){
			
			$shipwayCarrier = array();
			foreach($shipwayCarriers as $carrier){
				$shipwayCarrier[$carrier['carrier_id']]['id_shipway_carrier'] = $carrier['carrier_id'];
				$shipwayCarrier[$carrier['carrier_id']]['name'] = ucwords($carrier['name']);
			}
			
			$fields_form = array(
				'form' => array(
					'legend' => array(
						'title' => $this->l('Map store carrier to shipway carrier'),
						'icon' => 'icon-cogs'
					)
				),
			);
			
			foreach( $storeCarriers as $carrier){
				
				$fields_form['form']['input'][] = array(
					'type' => 'select',
					'label' => ($carrier['name']) ? $this->l( $carrier['name'] .': ') : 'My Store Prestashop',
					'name' => 'id_shipway_carrier['.$carrier['id_reference'].']',
					'options' => array(
						'default' => array('value' => 0, 'label' => $this->l('Choose shipway carrier')),
						'query' => $shipwayCarrier,
						'id' => 'id_shipway_carrier',
						'name' => 'name'
					),
				);
			}
			
			$fields_form['form']['submit'] = array(
				'title' => $this->l('Save'),
			);
			
			$helper = new HelperForm();
			$helper->show_toolbar = false;
			$helper->table = $this->table;
			$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
			$helper->default_form_language = $lang->id;
			$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;

			$helper->identifier = $this->identifier;
			$helper->submit_action = 'saveCarrierMap';
			$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name
				.'&tab_module='.$this->tab.'&module_name='.$this->name;
			$helper->token = Tools::getAdminTokenLite('AdminModules');
			$helper->tpl_vars = array(
				'fields_value' => $this->getConfigFieldsValues($storeCarriers),
				'languages' => $this->context->controller->getLanguages(),
				'id_language' => $this->context->language->id
			);
			
			return $helper->generateForm(array($fields_form));
			
		}
		
	}
	public function getConfigFieldsValues($storeCarriers = array() ){
		$shipCarrierMap = Configuration::get('id_shipway_carrier');
		$shipCarrierMap = (!empty($shipCarrierMap)) ? unserialize($shipCarrierMap) : array();
		$shipCarrierMap = ( empty($shipCarrierMap) ) ? Tools::getValue('id_shipway_carrier', array()) : $shipCarrierMap;

		$return_array =  array(
			'SHIPWAY_USERNAME' => Tools::getValue('SHIPWAY_USERNAME', Configuration::get('SHIPWAY_USERNAME')),
			'SHIPWAY_LINCENCE_KEY' => Tools::getValue('SHIPWAY_LINCENCE_KEY', Configuration::get('SHIPWAY_LINCENCE_KEY'))
		);
		
		foreach($storeCarriers as $carrier){
			$return_array['id_shipway_carrier[' .$carrier['id_reference']. ']'] = (isset($shipCarrierMap[$carrier['id_reference']])) ? $shipCarrierMap[$carrier['id_reference']] : '';
		}
		
		return $return_array;
	}
	protected function _postValidation()
	{
		if (Tools::isSubmit('saveShipwaySettings'))
		{
			if (!Tools::getValue('SHIPWAY_USERNAME'))
				$this->_postErrors[] = $this->l('Shipway Username is required.');
			elseif (!Tools::getValue('SHIPWAY_LINCENCE_KEY'))
				$this->_postErrors[] = $this->l('Shipway Licence Key is required.');
		}
	}
	protected function _postProcess(){
		if (Tools::isSubmit('saveShipwaySettings')){			
			if( $this->_addShipwayCarrier() ){
				Configuration::updateValue('SHIPWAY_USERNAME', Tools::getValue('SHIPWAY_USERNAME'));
				Configuration::updateValue('SHIPWAY_LINCENCE_KEY', Tools::getValue('SHIPWAY_LINCENCE_KEY'));
				$this->_html .= $this->displayConfirmation($this->l('Shipway settings updated'));
			}
		}elseif (Tools::isSubmit('saveCarrierMap')){			
			$id_shipway_carrier = Tools::getValue('id_shipway_carrier');
			
			Configuration::updateValue('id_shipway_carrier', serialize($id_shipway_carrier));
			$this->_html .= $this->displayConfirmation($this->l('Shipway carrier mapping saved.'));
			
			//echo "<pre>";print_r($id_shipway_carrier);die;
			
		}		
	}
	protected function _addShipwayCarrier(){
		$shipway_username = Tools::getValue('SHIPWAY_USERNAME');
		$shipway_password = Tools::getValue('SHIPWAY_LINCENCE_KEY');
		
		$url         = "http://shipway.in/api/getcarrier";
		$data_string = array(
			"username" => $shipway_username,
			"password" => $shipway_password
		);
		
		$data_string = json_encode($data_string);
		$curl        = curl_init();
		curl_setopt($curl, CURLOPT_HTTPHEADER, array(
			'Content-Type:application/json'
		));
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_POST, true);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $data_string);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		$output = curl_exec($curl);
		
		curl_close($curl);
		
		$output = (array)json_decode($output);
		
		if( isset($output['status']) && $output['status'] == 'Failed' ){
			$this->_html .= $this->displayError($this->l('Invalid shipway credentials. '));
			return false;
		}else{
			$db = Db::getInstance();
			
			$truncate_query = "TRUNCATE TABLE  `" . _DB_PREFIX_ . "sw_couriers`";
			$tableCreation = $db->execute($truncate_query);
			
			$insertQuery = '';
			foreach( $output as $id => $name){
				$insertQuery .= "('" .(int)$id. "','" . addslashes($name) . "','1'),";
			}
			$insertQuery = trim($insertQuery,', ');
			if($insertQuery){
				$insertQuery = "INSERT INTO `". _DB_PREFIX_."sw_couriers` (`carrier_id`,`name`,`status`) VALUES ".$insertQuery;
			}
			
			
			$tableCreation = $db->execute($insertQuery);
			
			if(!$tableCreation){
				$this->_html .= $this->displayError($this->l('Shipway carriers not fatched.Please contact us at contact@shipway.in '));
			}
			
		}
		
		return true;
	}
	public function hookActionAdminOrdersTrackingNumberUpdate( $params){

		$tracking_number = Tools::getValue('tracking_number');
		$username = Configuration::get('SHIPWAY_USERNAME');
		$password = Configuration::get('SHIPWAY_LINCENCE_KEY');
		
		$objOrder 		= ( isset($params['order']) ) ? $params['order'] : '';
		$objCarrier 	= ( isset($params['carrier']) ) ? $params['carrier'] : '' ;
		$objCustomer 	= ( isset($params['customer'])) ? $params['customer'] : '';
		$objAddress 	= new Address($objOrder->id_address_invoice, $this->context->language->id);
		$objState 		= new State($objAddress->id_state);
		
		$carrier_id 	= ( isset($objOrder->id_carrier) ) ? $objOrder->id_carrier : 0;
		
		$objCarrier 	= new Carrier($carrier_id);
		$reference_id 	= ( isset($objCarrier->id_reference) ) ? $objCarrier->id_reference : 0;
		
		$shipCarrierMap = Configuration::get('id_shipway_carrier');
		$shipCarrierMap = (!empty($shipCarrierMap)) ? unserialize($shipCarrierMap) : array();
		
		$shipway_carrier_id = 0;
		if(!empty($shipCarrierMap)){
			$shipway_carrier_id = ( isset($shipCarrierMap[$reference_id]) ) ? (int)$shipCarrierMap[$reference_id] : 0;
		}
				
		$shipwayData = array();
		$shipwayData['order_id'] 	= ( isset($objOrder->id) ) ? $objOrder->id : '';
		$shipwayData['awb'] 		= $tracking_number;
		$shipwayData['carrier_id'] 	= $shipway_carrier_id;
		$shipwayData['username'] 	= $username;
		$shipwayData['password'] 	= $password;
		
		$orders = array();
		$orders['order_id'] 	= ( isset($objOrder->id) ) ? $objOrder->id : '';
		$orders['order_date']	= ( isset($objOrder->date_add) ) ? $objOrder->date_add : '';
		$orders['firstname']	= ( isset($objCustomer->firstname) ) ? $objCustomer->firstname : '';
		$orders['lastname']		= ( isset($objCustomer->lastname) ) ? $objCustomer->lastname : '';
		$orders['email']		= ( isset($objCustomer->email) ) ? $objCustomer->email : '';
		$orders['phone']		= ( isset($objAddress->phone) ) ? $objAddress->phone : '';
		$orders['address']		= ( isset($objAddress->address1) ) ? $objAddress->address1 : '';
		$orders['address']		.= ( isset($objAddress->address2) ) ? ','.$objAddress->address2 : '';
		$orders['city']			= ( isset($objAddress->city) ) ?$objAddress->city : '';
		$orders['state']		= ( isset($objState->name) ) ? $objState->name : '';
		$orders['zipcode']		= ( isset($objAddress->postcode) ) ? $objAddress->postcode : '';
		$orders['country']		= ( isset($objAddress->country) ) ? $objAddress->country : '' ;
		$orders['amount']		= ( isset($objOrder->total_paid) ) ? $objOrder->total_paid : 0;
		
		$orders['payment_code']			= ( isset($objOrder->module) ) ? $objOrder->module : '';
		$orders['payment_type']			= ( isset($objOrder->module) && $objOrder->module != 'cashondelivery') ? 'P' : 'C' ;
		$orders['collectable_amount']	= ( $orders['payment_type'] == 'C') ? $orders['amount'] : 0;
		$orders['return_address']		= preg_replace( "/\r|\n/", "", Configuration::get('BLOCKCONTACTINFOS_ADDRESS') ).' Email: '.Configuration::get('BLOCKCONTACTINFOS_EMAIL');
		
		$shipwayData['email'] 		= ( isset($orders['email']) ) ? $orders['email'] : '';
		$shipwayData['phone'] 		= ( isset($orders['phone']) ) ? $orders['phone'] : '';
		$shipwayData['first_name'] 	= ( isset($orders['firstname']) ) ? $orders['firstname'] : '';
		$shipwayData['last_name'] 	= ( isset($orders['lastname']) ) ? $orders['lastname'] : '';
		$shipwayData['company'] 	= Configuration::get('BLOCKCONTACTINFOS_COMPANY');
		$shipwayData['phone'] 		= $orders['phone'];
		
		$products = array();
		$strProducts = '';
		
		$productList = $objOrder->getProducts();
		foreach($productList as $key => $product){
			
			$objProduct = new Product( $product['product_id'] );
			
			$is_rewrite_active = (bool)Configuration::get('PS_REWRITING_SETTINGS');
			$preview_url = $this->context->link->getProductLink(
				$objProduct,
				$this->getFieldValue($objProduct, 'link_rewrite', $this->context->language->id),
				Category::getLinkRewrite($this->getFieldValue($objProduct, 'id_category_default'), $this->context->language->id),
				null,
				null,
				(int)Context::getContext()->shop->id,
				0,
				$is_rewrite_active
			);
					
			
			$products[$key]['product_id'] 	= ( isset($product['product_id']) ) ? $product['product_id'] : '';
			$products[$key]['name'] 		=  ( isset($product['product_name']) ) ? $product['product_name'] : '';
			$products[$key]['quantity'] 	= ( isset($product['product_quantity'])) ? $product['product_quantity'] : '';
			$products[$key]['price'] 		= ( isset( $product['product_price'] )) ? $product['product_price'] : '';
			$products[$key]['url'] = $preview_url;
			
			$strProducts .= ','.$product['product_name'];
		}
		
		$strProducts = substr( trim($strProducts , ', ') , 0 ,200) ;
		$shipwayData['products'] = $strProducts;
		$orders['products'] = $products;
		$shipwayData['order'] = $orders;
		
		$this->pushShipwayOrder($shipwayData);
	}
	public function pushShipwayOrder($data){
		
		$strValidate = 'username|password|awb|order_id|carrier_id';
		
		$error = array();
		foreach(explode('|',$strValidate) as $field){
			if( !isset($data[$field]) || empty($data[$field]) ){
				$error[] = $field;
			}
		}
		
		if(!empty($error)){
			return;
		}
		$url = "http://shipway.in/api/pushOrderData";
			
		$data_string = json_encode($data);
		
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_HTTPHEADER, array(
			'Content-Type:application/json'
		));
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_POST, true);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $data_string);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		
		$output = curl_exec($curl);
		$output = json_decode($output);
		curl_close($curl);
		
		$message = (isset($output->message)) ? $output->message : 'Something went wrong.';
		
		if(isset($output->status) && strtolower($output->status) == 'success'){
			//success notification
		}else{
			//Error Notification
		}			
	}
	public function getFieldValue($obj, $key, $id_lang = null){
        if ($id_lang) {
            $default_value = (isset($obj->id) && $obj->id && isset($obj->{$key}[$id_lang])) ? $obj->{$key}[$id_lang] : false;
        } else {
            $default_value = isset($obj->{$key}) ? $obj->{$key} : false;
        }

        return Tools::getValue($key.($id_lang ? '_'.$id_lang : ''), $default_value);
    }
}